from typing import Any
from django.shortcuts import render, get_object_or_404, redirect
from django.views import generic

from webapp.forms import ArticleForm
from webapp.models import Article


# CRUD - Create Read Update Delete


class IndexRedirectView(generic.RedirectView):
    pattern_name = 'index'


class IndexView(generic.View):
    def get(self, request):
        context = {'articles': Article.objects.all()}

        return render(request, 'index.html', context)    


class ArticleDetailView(generic.TemplateView):
    template_name = 'articles/detail.html'

    def get_context_data(self, **kwargs):
        kwargs['article'] = get_object_or_404(Article, id=kwargs['id'])
        return super().get_context_data(**kwargs)


def article_create_view(request):
    if request.method == 'GET':
        form = ArticleForm()
        return render(request, 'articles/create.html', context={'form': form})

    elif request.method == 'POST':
        form = ArticleForm(data=request.POST)

        if form.is_valid():
            tags = form.cleaned_data.pop('tags')

            article = Article.objects.create(**form.cleaned_data)
            article.tags.set(tags)

            return redirect('articles_detail', id=article.id)
    
        else:            
           return render(
               request, 'articles/create.html', 
               context={'form': form},
            ) 


def article_update_view(request, id):
    article = get_object_or_404(Article, id=id)
    
    if request.method == 'GET':
        form = ArticleForm(initial={
            'title': article.title,
            'text': article.text,
            'author': article.author,
            'tags': article.tags.all(),
        })

        return render(
            request, 'articles/update.html', 
            context={'article': article, 'form': form},
        )
    
    elif request.method == 'POST':
        form = ArticleForm(data=request.POST)
        
        if form.is_valid():
            tags = form.cleaned_data.pop('tags')

            article.title = form.cleaned_data.get('title')
            article.author = form.cleaned_data.get('author')
            article.text = form.cleaned_data.get('text')

            article.save()
            
            article.tags.set(tags)

            return redirect('index')
        else:
            return render(request, 'articles/update.html', context={'form': form, 'article': article})


def article_delete_view(request, id):
    get_object_or_404(Article, id=id).delete()
    
    return redirect('index')
